@@ -43,9 +43,10 @@ import time
 from multiprocessing import Process
 from pprint import pprint
 
-from pysimsearch import doc_reader
+from pysimsearch import term_vec
 from pysimsearch.sim_index import MemorySimIndex
 from pysimsearch.sim_index import ShelfSimIndex
+from pysimsearch.sim_index import ConcurrentSimIndex
 from pysimsearch.sim_index import SimIndexCollection
 from pysimsearch.sim_index import RemoteSimIndex
 from pysimsearch import sim_server
@@ -67,20 +68,16 @@ class SimIndexTest(object):
     def setUp(self):
         with io.StringIO(self.stopfile_buffer) as stopfile:
             self.sim_index.load_stoplist(stopfile)
-        
-        named_files = ((docname, io.StringIO(doc))
-                            for (docname, doc) in self.docs)
-        self.sim_index.index_files(named_files)
-    
-
+            
+        self.sim_index.index_string_buffers(self.docs)
 
     # Stopword list
     stopfile_buffer = "stopword1 stopword2"
 
     # Test documents
-    docs = [ ('doc1', "hello there world     hello stopword1"),
+    docs = ( ('doc1', "hello there world     hello stopword1"),
              ('doc2', "hello       world           stopword2"),
-             ('doc3', "hello there       bob      ") ]
+             ('doc3', "hello there       bob      ") )
     
     # Postings that correspond to test documents
     golden_postings = { 'hello': {'doc1': 2, 'doc2': 1, 'doc3': 1},
@@ -174,7 +171,7 @@ class SimIndexTest(object):
         # We unpack the golden hit lists, construct a golden set of docnames
         # for the hits, and compare with sim_index.docnames_with_terms()
         for (query, golden_doc_hits) in self.golden_conj_hits.items():
-            query_vec = doc_reader.term_vec_from_string(query)
+            query_vec = term_vec.term_vec(query)
             terms = [term for (term, freq) in query_vec.items()]
             
             self.assertEqual(golden_doc_hits,
@@ -188,7 +185,7 @@ class SimIndexTest(object):
         self.sim_index.set_query_scorer('simple_count')
         for (query, golden_doc_hits) in self.golden_scored_hits.items():
             self.assertEqual(golden_doc_hits,
-                             dict(self.sim_index.query_by_string(query)),
+                             dict(self.sim_index.query(query)),
                              msg = "query={}".format(query))
 
     def test_query_tfidf_scorer(self):
@@ -198,12 +195,73 @@ class SimIndexTest(object):
         '''
         self.sim_index.set_query_scorer('tfidf')
         for (query, golden_doc_hits_cos) in self.get_golden_hits_cos().items():
-            results = self.sim_index.query_by_string(query)
+            results = self.sim_index.query(query)
             for (docname, score) in results:
                 self.assertAlmostEqual(score,
                                        golden_doc_hits_cos[docname],
                                        msg="results={}".format(str(results)))
+    
+    def test_del_docids(self):
+        '''Test del_docids()'''
+        retest_list = (self.test_docnames_with_terms,
+                       self.test_query_simple_scorer,
+                       self.test_query_tfidf_scorer,)
+        
+        # Make sure that the selected tests already pass (just for clarity)
+        for test in retest_list:
+            test()
 
+        # Add an extra doc to the index
+        self.sim_index.index_string_buffers( (('extra_doc', "hello world"),) )
+
+        # Make sure that selected tests fail when we add an extra 'unexpected'
+        # doc to the index
+        for test in retest_list:
+            self.assertRaises(AssertionError, test)
+        
+        # Delete the extra doc
+        docid = self.sim_index.name_to_docid('extra_doc')
+        print('extra docid={}'.format(docid))
+        self.sim_index.del_docids(docid)
+
+        # Now make sure that the selected tests pass again
+        for test in retest_list:
+            test()
+    
+    def test_config(self):
+        '''Ensure that various config params are properly handled'''
+
+        ### Test 'lowercase' param
+        
+        def _check_lc(index, golden_results):
+            '''helper that checks index against golden_results'''
+            for (term, golden_docs) in golden_results:
+                self.assertEqual(
+                    set(index.docnames_with_terms(term)), golden_docs)
+                self.assertEqual(
+                    set([doc for (doc, score) in index.query(term)]), golden_docs)
+                
+        # test data
+        test_docs = (('doc1', 'Hello There'),
+                     ('doc2', 'hello there'))
+
+        # lowercase=True
+        index = MemorySimIndex()
+        index.set_config('lowercase', True)
+        index.index_string_buffers(test_docs)
+        golden_results = (('hello', {'doc1', 'doc2'}),
+                          ('Hello', {'doc1', 'doc2'}),
+                          ('HELLO', {'doc1', 'doc2'}))
+        _check_lc(index, golden_results)
+        
+        # lowercase=False
+        index = MemorySimIndex()
+        index.set_config('lowercase', False)
+        index.index_string_buffers(test_docs)
+        golden_results = (('hello', {'doc2'}),
+                          ('Hello', {'doc1'}),
+                          ('HELLO', set()))
+        _check_lc(index, golden_results)
 
 class MemorySimIndexTest(SimIndexTest, unittest.TestCase):
     '''
@@ -246,6 +304,21 @@ class ShelfSimIndexTest(SimIndexTest, unittest.TestCase):
     def tearDown(self):
         self.sim_index.close()
 
+class ConcurrentSimIndexTest(SimIndexTest, unittest.TestCase):
+    '''
+    All tests hitting the SimIndex interface are in the parent class, SimIndexTest
+    
+    Tests for api's not in parent class are tested separately here.  This is
+    so we can reuse test code across all implementations of SimIndex.
+    '''
+    
+    def setUp(self):
+        print("ConcurrentSimIndexTest")
+        self.sim_index = ConcurrentSimIndex(MemorySimIndex())
+        super(ConcurrentSimIndexTest, self).setUp()
+
+    def tearDown(self):
+        pass
 
 class SimIndexCollectionTest(SimIndexTest, unittest.TestCase):
     '''
@@ -265,7 +338,7 @@ class SimIndexCollectionTest(SimIndexTest, unittest.TestCase):
     
     def tearDown(self):
         pass
-
+    
 
 class SimIndexRemoteCollectionTest(SimIndexTest, unittest.TestCase):
     '''
@@ -284,8 +357,6 @@ class SimIndexRemoteCollectionTest(SimIndexTest, unittest.TestCase):
 
         print("SimIndexRemoteCollectionTest")
         
-        self.sim_index = SimIndexCollection()
-        
         # We will create a collection tree of the form:
         #
         #      Root
@@ -294,26 +365,53 @@ class SimIndexRemoteCollectionTest(SimIndexTest, unittest.TestCase):
         #   /\     /\
         #  1  2   3  4
         self.processes = []
+
+        # start leaves
         for i in range(4):
             port = 9100 + i
             process = Process(target=sim_server.start_sim_index_server,
                               kwargs={'port': port, 'logRequests': False})
             process.daemon = True
+            process.start()
             self.processes.append(process)
             
-        for process in self.processes:
-            process.start()
-            
-        print("Waiting for servers to start")
+        print("Waiting for leaf servers to start")
         time.sleep(0.1)
-
-        sim_index_nodes = (SimIndexCollection(), SimIndexCollection())  # (A, B)
+        
+        leaf_nodes = [[],[]]
         for i in range(4):
             port = 9100 + i
-            sim_index_nodes[i//2].add_shards(
+            leaf_nodes[i//2].append(RemoteSimIndex(
+                "http://localhost:{}/RPC2".format(port)))
+
+        # start interior nodes (A, B)
+        for i in range(2):
+            port = 9200 + i
+            process = Process(
+                target=sim_server.start_sim_index_server,
+                kwargs={ 'port': port,
+                         'backends': leaf_nodes[i],
+                         'root': False,
+                         'logRequests': False
+                        }
+            )
+            process.daemon = True
+            process.start()
+            self.processes.append(process)
+
+        print("Waiting for intermediate servers to start")
+        time.sleep(0.1)        
+
+        interior_nodes = []
+        for i in range(2):
+            port = 9200 + i
+            interior_nodes.append(
                 RemoteSimIndex("http://localhost:{}/RPC2".format(port)))
 
-        self.sim_index.add_shards(*sim_index_nodes)
+        # root node
+        self.sim_index = SimIndexCollection(root=True)
+        self.sim_index.add_shards(*interior_nodes)
+        
         super(SimIndexRemoteCollectionTest, self).setUp()
     
     def tearDown(self):
